@@ -5,7 +5,7 @@
 #' @description Check parameter values and save as named vector.
 #'
 #' @details  Please do not increasing the number of SNPs or individuals beyond
-#'  the numbers present in the datasets, as this may cause R to crash.
+#'   the numbers present in the datasets, as this may cause R to crash.
 #'
 #' @param GenoM matrix with genotype data, size nInd x nSnp
 #' @param LifeHistData Dataframe with 3 columns:
@@ -19,26 +19,27 @@
 #'   (up to 42).
 #' @param Err Estimated genotyping error rate.
 #' @param MaxMismatch Maximum number of loci at which candidate parent and
-#'  offspring are allowed to be opposite homozygotes, or be excluded.
-#' @param Tfilter Threshold log-likelihood ratio between a proposed
-#'   relationship versus unrelated, to select candidate relatives. Typically a
-#'   negative value, related to the fact that unconditional likelihoods are
-#'   calculated during the filtering steps. More negative values may decrease
+#'   offspring are allowed to be opposite homozygotes, or be excluded.
+#' @param Tfilter Threshold log-likelihood ratio between a proposed relationship
+#'   versus unrelated, to select candidate relatives. Typically a negative
+#'   value, related to the fact that unconditional likelihoods are calculated
+#'   during the filtering steps. More negative values may decrease
 #'   non-assignment, but will increase computational time.
 #' @param Tassign Minimum log-likelihood ratio required for acceptance of
-#'  proposed relationship, relative to next most likely relationship. Higher
-#'  values result in more conservative assignments.
-#' @param MaxSibshipSize  Maximum number of offspring for a single individual
-#'  (a generous safety margin is advised).
-#' @param DummyPrefix character vector of length 2 with prefixes for dummy
-#'   dams (mothers) and sires (fathers); maximum 20 characters each.
+#'   proposed relationship, relative to next most likely relationship. Higher
+#'   values result in more conservative assignments.
+#' @param MaxSibshipSize  Maximum number of offspring for a single individual (a
+#'   generous safety margin is advised).
+#' @param DummyPrefix character vector of length 2 with prefixes for dummy dams
+#'   (mothers) and sires (fathers); maximum 20 characters each.
 #' @param Complexity Either "full" (default), "simp" (no explicit consideration
 #'   of inbred relationships), "mono" (monogamous breeding system), or "herm"
 #'   (hermaphrodites)
 #' @param FindMaybeRel  Identify pairs of non-assigned likely relatives after
 #'   pedigree reconstruction. Can be time-consuming in large datasets.
 #' @param CalcLLR  Calculate log-likelihood ratios for all assignments. Can be
-#'  time-consuming in large datasets.
+#'   time-consuming in large datasets.
+#'
 #' @return A 1-row dataframe with parameter values
 #'
 #' @keywords internal
@@ -108,44 +109,29 @@ SeqPrep <- function(GenoM = NULL,
 }
 
 
-
 #=======================================================================
-#' @title check GenoM
+#' @title check LifeHistData
 #'
-#' @description Check that the provided genotype matrix is in the correct format
+#' @description Check that the provided LifeHistData is in the correct format
 #'
-#' @param GenoM the genotype matrix
+#' @param LifeHistData the dataframe with ID - Sex - Birth year
 #'
-#' @return updates GenoM internally to remove low call rate individuals (<.5)
-#'   and low call rate SNPs (<0.1), if necessary
+#' @return a dataframe with corrected formatting of LifeHistData, if required
 #'
 #' @keywords internal
 
-CheckGeno <- function(GenoM) {
-	if (is.null(GenoM)) stop("please provide 'GenoM'")
-  if (!is.matrix(GenoM)) stop("'GenoM' should be a numeric matrix")
-	if (!all(GenoM %in% c(0,1,2,-9))) stop("'GenoM' not in <seq> format, please use GenoConvert")
-
-  if (is.null(rownames(GenoM))) stop("'GenoM' has no rownames, these should be the individual IDs")
-	if (any(duplicated(rownames(GenoM))))  stop("'GenoM' has duplicate IDs. Please exclude or rename these samples, or run GenoConvert with UseFID=TRUE.")
-
-  Excl <- list()
-	Lscored <- apply(GenoM, 1, function(x) sum(x!=-9))
-  if (any(Lscored < ncol(GenoM)/2)) {
-    warning(paste("There are ", sum(Lscored < ncol(GenoM)/2)," individuals scored for <50% of SNPs, these will be excluded"),
-            immediate.=TRUE)
-		GenoM <<- GenoM[Lscored >= ncol(GenoM)/2, ]
-		Excl[["ExcludedInd"]] <- rownames(GenoM)[which(Lscored < ncol(GenoM)/2)]
+CheckLH <- function(LifeHistData) {
+  if ((sum(LifeHistData[,2] %in% c(1,2,4)) < nrow(LifeHistData)/10) &
+      (sum(is.na(LifeHistData[,2])) + sum(LifeHistData[,2]==3, na.rm=TRUE) < nrow(LifeHistData)/2)) {
+    stop("LifeHistData column 2 should contain Sex coded as 1=female, 2=male, 3/NA=unknown, 4=hermaphrodite")
   }
+  names(LifeHistData)[1:3] <- c("ID", "Sex", "BY")
+  LifeHistData$ID <- as.character(LifeHistData$ID)
+  LifeHistData <- LifeHistData[!is.na(LifeHistData$ID), ]
+  for (x in c("Sex", "BY")) LifeHistData[, x] <- as.integer(LifeHistData[, x])
+  LifeHistData$BY[is.na(LifeHistData$BY)] <- -999
+  LifeHistData$Sex[is.na(LifeHistData$Sex)] <- 3
+  LifeHistData$Sex[!LifeHistData$Sex %in% 1:4] <- 3
 
-  Nscored <- apply(GenoM, 2, function(x) sum(x!=-9))
-  if (any(Nscored < nrow(GenoM)/10)) {
-    warning(paste("There are ", sum(Nscored < nrow(GenoM)/10)," SNPs scored for <10% of individuals, these will be excluded"),
-            immediate.=TRUE)
-		GenoM <<- GenoM[, Nscored >= ncol(GenoM)/10]
-		Excl[["ExcludedSnps"]] <- which(Nscored < nrow(GenoM)/10)
-  }
-  Excl
+  return( LifeHistData )
 }
-
-
